/*****************************************************************************
 * Open MCT, Copyright (c) 2014-2018, United States Government
 * as represented by the Administrator of the National Aeronautics and Space
 * Administration. All rights reserved.
 *
 * Open MCT is licensed under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 *
 * Open MCT includes source code licensed under additional open source
 * licenses. See the Open Source Licenses file (LICENSES.md) included with
 * this source code distribution or the Licensing information page available
 * at runtime from the About dialog for additional information.
 *****************************************************************************/

define(
    ["../src/TelemetryHandler"],
    function (TelemetryHandler) {

        describe("The telemetry handler", function () {
            // TelemetryHandler just provides a factory
            // for TelemetryHandle, so most real testing
            // should happen there.
            var mockQ,
                mockSubscriber,
                mockDomainObject,
                mockCallback,
                mockSubscription,
                handler;

            beforeEach(function () {
                mockQ = jasmine.createSpyObj("$q", ["when"]);
                mockSubscriber = jasmine.createSpyObj(
                    'telemetrySubscriber',
                    ['subscribe']
                );
                mockDomainObject = jasmine.createSpyObj(
                    'domainObject',
                    ['getId', 'getCapability']
                );
                mockCallback = jasmine.createSpy('callback');
                mockSubscription = jasmine.createSpyObj(
                    'subscription',
                    [
                        'unsubscribe',
                        'getTelemetryObjects',
                        'getRangeValue',
                        'getDomainValue'
                    ]
                );

                mockSubscriber.subscribe.and.returnValue(mockSubscription);

                handler = new TelemetryHandler(mockQ, mockSubscriber);
            });

            it("acts as a factory for subscription objects", function () {
                var handle = handler.handle(
                    mockDomainObject,
                    mockCallback
                );
                // Just verify that this looks like a TelemetrySubscription
                [
                    "unsubscribe",
                    "getTelemetryObjects",
                    "getRangeValue",
                    "getDomainValue",
                    "request"
                ].forEach(function (method) {
                    expect(handle[method]).toEqual(jasmine.any(Function));
                });
            });

        });
    }
);
